@@ -1,13 +1,12 @@
 #
 #    Uncomplicated VM Builder
-#    Copyright (C) 2007-2008 Canonical Ltd.
+#    Copyright (C) 2007-2009 Canonical Ltd.
 #    
 #    See AUTHORS for list of contributors
 #
 #    This program is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    it under the terms of the GNU General Public License version 3, as
+#    published by the Free Software Foundation.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,14 +19,25 @@
 #    Logging
 
 import logging
+import os
+import tempfile
 
-FORMAT='%(asctime)s %(levelname)-8s %(message)s'
-logging.basicConfig(format=FORMAT, level=logging.INFO)
+format = '%(asctime)s %(levelname)-8s: %(message)s'
+
+fd, logfile = tempfile.mkstemp()
+
+# Log everything to the logfile
+logging.basicConfig(format=format, level=logging.DEBUG, datefmt='%Y-%m-%d %H:%M', stream=os.fdopen(fd, 'a+'), filemode='w')
+
+console = logging.StreamHandler()
+console.setLevel(logging.INFO)
+console.setFormatter(logging.Formatter(format))
+logging.getLogger('').addHandler(console)
 
 def set_verbosity(option, opt_str, value, parser):
     if opt_str == '--debug':
-        logging.getLogger().setLevel(logging.DEBUG)
+        console.setLevel(logging.DEBUG)
     elif opt_str == '--verbose':
-        logging.getLogger().setLevel(logging.INFO)
+        console.setLevel(logging.INFO)
     elif opt_str == '--quiet':
-        logging.getLogger().setLevel(logging.CRITICAL)
+        console.setLevel(logging.CRITICAL)
