@@ -1,13 +1,12 @@
 #
 #    Uncomplicated VM Builder
-#    Copyright (C) 2007-2008 Canonical Ltd.
+#    Copyright (C) 2007-2009 Canonical Ltd.
 #    
 #    See AUTHORS for list of contributors
 #
 #    This program is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    it under the terms of the GNU General Public License version 3, as
+#    published by the Free Software Foundation.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +22,7 @@ import os
 import suite
 import shutil
 import socket
+import tempfile
 import VMBuilder
 import VMBuilder.disk as disk
 from   VMBuilder.util import run_cmd
@@ -36,6 +36,10 @@ class Dapper(suite.Suite):
     disk_prefix = 'hd'
     xen_kernel_flavour = None
     virtio_net = False
+    chpasswd_cmd = [ 'chpasswd', '--md5' ]
+
+    def pre_install(self):
+        pass
 
     def check_kernel_flavour(self, arch, flavour):
         return flavour in self.valid_flavours[arch]
@@ -49,9 +53,14 @@ class Dapper(suite.Suite):
         logging.debug("debootstrapping")
         self.debootstrap()
 
+        self.pre_install()
+
         logging.debug("Setting up sources.list")
         self.install_sources_list()
 
+        logging.debug("Setting up apt proxy")
+        self.install_apt_proxy()
+
         logging.debug("Installing fstab")
         self.install_fstab()
 
@@ -68,6 +77,9 @@ class Dapper(suite.Suite):
         logging.debug("Preventing daemons from starting")
         self.prevent_daemons_starting()
 
+        logging.debug('Binding /dev and /proc filesystems')
+        self.mount_dev_proc()
+
         if self.vm.hypervisor.needs_bootloader:
             logging.debug("Installing menu.list")
             self.install_menu_lst()
@@ -78,32 +90,53 @@ class Dapper(suite.Suite):
             logging.debug("Creating device.map")
             self.install_device_map()
 
-        logging.debug("Installing ssh keys")
-        self.install_authorized_keys()
-
         logging.debug("Installing extra packages")
         self.install_extras()
 
         logging.debug("Creating initial user")
         self.create_initial_user()
 
+        logging.debug("Installing ssh keys")
+        self.install_authorized_keys()
+
         logging.debug("Copy host settings")
         self.copy_settings()
 
+        logging.debug("Setting timezone")
+        self.set_timezone()
+
         logging.debug("Making sure system is up-to-date")
         self.update()
 
         logging.debug("Setting up final sources.list")
         self.install_sources_list(final=True)
 
+        logging.debug("cleaning apt")
+        self.run_in_target('apt-get', 'clean');
+
         logging.debug("Unmounting volatile lrm filesystems")
         self.unmount_volatile()
 
+        logging.debug('Unbinding /dev and /proc filesystems')
+        self.unmount_dev_proc()
+
+        if hasattr(self.vm, 'ec2') and self.vm.ec2:
+            logging.debug("Configuring for ec2")
+            self.install_ec2()
+
         logging.debug("Unpreventing daemons from starting")
         self.unprevent_daemons_starting()
 
+        if self.vm.manifest:
+            logging.debug("Creating manifest")
+            manifest_contents = self.run_in_target('dpkg-query', '-W', '--showformat=${Package} ${Version}\n')
+            fp = open(self.vm.manifest, 'w')
+            fp.write(manifest_contents)
+            fp.close
+
     def update(self):
-        self.run_in_target('apt-get', '-y', '--force-yes', 'dist-upgrade')
+        self.run_in_target('apt-get', '-y', '--force-yes', 'dist-upgrade',
+                           env={ 'DEBIAN_FRONTEND' : 'noninteractive' })
         
     def install_authorized_keys(self):
         if self.vm.ssh_key:
@@ -114,14 +147,47 @@ class Dapper(suite.Suite):
             os.mkdir('%s/home/%s/.ssh' % (self.destdir, self.vm.user), 0700)
             shutil.copy(self.vm.ssh_user_key, '%s/home/%s/.ssh/authorized_keys' % (self.destdir, self.vm.user))
             os.chmod('%s/home/%s/.ssh/authorized_keys' % (self.destdir, self.vm.user), 0644)
+            self.run_in_target('chown', '-R', '%s:%s' % (self.vm.user,)*2, '/home/%s/.ssh/' % (self.vm.user)) 
+
         if self.vm.ssh_user_key or self.vm.ssh_key:
             if not self.vm.addpkg:
                 self.vm.addpkg = []
             self.vm.addpkg += ['openssh-server']
 
+    def mount_dev_proc(self):
+        run_cmd('mount', '--bind', '/dev', '%s/dev' % self.destdir)
+        self.vm.add_clean_cmd('umount', '%s/dev' % self.destdir, ignore_fail=True)
+
+        run_cmd('mount', '--bind', '/dev/pts', '%s/dev/pts' % self.destdir)
+        self.vm.add_clean_cmd('umount', '%s/dev/pts' % self.destdir, ignore_fail=True)
+
+        self.run_in_target('mount', '-t', 'proc', 'proc', '/proc')
+        self.vm.add_clean_cmd('umount', '%s/proc' % self.destdir, ignore_fail=True)
+
+    def unmount_dev_proc(self):
+    	run_cmd('umount', '%s/dev/pts' % self.destdir)
+        run_cmd('umount', '%s/dev' % self.destdir)
+        run_cmd('umount', '%s/proc' % self.destdir)
+
+    def update_passwords(self):
+        # Set the user password, using md5
+        self.run_in_target(stdin=('%s:%s\n' % (self.vm.user, getattr(self.vm, 'pass'))), *self.chpasswd_cmd)
+
+        # Lock root account only if we didn't set the root password
+        if self.vm.rootpass:
+            self.run_in_target(stdin=('%s:%s\n' % ('root', self.vm.rootpass)), *self.chpasswd_cmd)
+        else:
+            self.run_in_target('usermod', '-L', 'root')
+
+        if self.vm.lock_user:
+            logging.info('Locking %s' %(self.vm.user))
+            self.run_in_target('usermod', '-L', self.vm.user)
+
     def create_initial_user(self):
-        self.run_in_target('adduser', '--disabled-password', '--gecos', self.vm.name, self.vm.user)
-        self.run_in_target('chpasswd', stdin=('%s:%s\n' % (self.vm.user, getattr(self.vm, 'pass'))))
+        if self.vm.uid:
+            self.run_in_target('adduser', '--disabled-password', '--uid', self.vm.uid, '--gecos', self.vm.name, self.vm.user)
+        else:
+            self.run_in_target('adduser', '--disabled-password', '--gecos', self.vm.name, self.vm.user)
         self.run_in_target('addgroup', '--system', 'admin')
         self.run_in_target('adduser', self.vm.user, 'admin')
 
@@ -129,8 +195,7 @@ class Dapper(suite.Suite):
         for group in ['adm', 'audio', 'cdrom', 'dialout', 'floppy', 'video', 'plugdev', 'dip', 'netdev', 'powerdev', 'lpadmin', 'scanner']:
             self.run_in_target('adduser', self.vm.user, group, ignore_fail=True)
 
-        # Lock root account
-        self.run_in_target('chpasswd', '-e', stdin='root:!\n')
+        self.update_passwords()
 
     def kernel_name(self):
         return 'linux-image-%s' % (self.vm.flavour or self.default_flavour[self.vm.arch],)
@@ -160,20 +225,11 @@ class Dapper(suite.Suite):
             run_cmd('umount', mntpnt)
 
     def install_menu_lst(self):
-        run_cmd('mount', '--bind', '/dev', '%s/dev' % self.destdir)
-        self.vm.add_clean_cmd('umount', '%s/dev' % self.destdir, ignore_fail=True)
-
-        self.run_in_target('mount', '-t', 'proc', 'proc', '/proc')
-        self.vm.add_clean_cmd('umount', '%s/proc' % self.destdir, ignore_fail=True)
-
         self.run_in_target(self.updategrub, '-y')
         self.mangle_grub_menu_lst()
         self.run_in_target(self.updategrub)
         self.run_in_target('grub-set-default', '0')
 
-        run_cmd('umount', '%s/dev' % self.destdir)
-        run_cmd('umount', '%s/proc' % self.destdir)
-
     def mangle_grub_menu_lst(self):
         bootdev = disk.bootpart(self.vm.disks)
         run_cmd('sed', '-ie', 's/^# kopt=root=\([^ ]*\)\(.*\)/# kopt=root=\/dev\/hd%s%d\\2/g' % (bootdev.disk.devletters(), bootdev.get_index()+1), '%s/boot/grub/menu.lst' % self.destdir)
@@ -193,6 +249,10 @@ class Dapper(suite.Suite):
         # final vm is going to be on).
         self.run_in_target('apt-get', 'update', ignore_fail=final)
 
+    def install_apt_proxy(self):
+        if self.vm.proxy is not None:
+            self.vm.install_file('/etc/apt/apt.conf', '// Proxy added by vmbuilder\nAcquire::http { Proxy "%s"; };' % self.vm.proxy)
+
     def install_fstab(self):
         if self.vm.hypervisor.preferred_storage == VMBuilder.hypervisor.STORAGE_FS_IMAGE:
             self.install_from_template('/etc/fstab', 'dapper_fstab_fsimage', { 'fss' : disk.get_ordered_filesystems(self.vm), 'prefix' : self.disk_prefix })
@@ -203,12 +263,18 @@ class Dapper(suite.Suite):
         self.install_from_template('/boot/grub/device.map', 'devicemap', { 'prefix' : self.disk_prefix })
 
     def debootstrap(self):
-        cmd = ['/usr/sbin/debootstrap', '--arch=%s' % self.vm.arch, self.vm.suite, self.destdir, self.debootstrap_mirror()]
-        run_cmd(*cmd)
+        cmd = ['/usr/sbin/debootstrap', '--arch=%s' % self.vm.arch]
+        if self.vm.variant:
+            cmd += ['--variant=%s' % self.vm.variant]
+        cmd += [self.vm.suite, self.destdir, self.debootstrap_mirror()]
+        kwargs = { 'env' : { 'DEBIAN_FRONTEND' : 'noninteractive' } }
+        if self.vm.proxy:
+            kwargs['env']['http_proxy'] = self.vm.proxy
+        run_cmd(*cmd, **kwargs)
     
     def debootstrap_mirror(self):
         if self.vm.iso:
-            os.mkdir(isodir)
+            isodir = tempfile.mkdtemp()
             self.vm.add_clean_cb(lambda:os.rmdir(isodir))
             run_cmd('mount', '-o', 'loop', '-t', 'iso9660', self.vm.iso, isodir)
             self.vm.add_clean_cmd('umount', isodir)
@@ -220,9 +286,7 @@ class Dapper(suite.Suite):
 
 
     def install_mirrors(self):
-        if self.vm.iso:
-            mirror = "file:///isomnt"
-        elif self.vm.install_mirror:
+        if self.vm.install_mirror:
             mirror = self.vm.install_mirror
         else:
             mirror = self.vm.mirror
@@ -245,7 +309,7 @@ class Dapper(suite.Suite):
 
     def install_grub(self):
         self.run_in_target('apt-get', '--force-yes', '-y', 'install', 'grub')
-        run_cmd('cp', '-a', '%s%s/%s/' % (self.destdir, self.grubroot, self.vm.arch == 'amd64' and 'x86_64-pc' or 'i386-pc'), '%s/boot/grub' % self.destdir) 
+        run_cmd('rsync', '-a', '%s%s/%s/' % (self.destdir, self.grubroot, self.vm.arch == 'amd64' and 'x86_64-pc' or 'i386-pc'), '%s/boot/grub/' % self.destdir) 
 
     def create_devices(self):
         import VMBuilder.plugins.xen
@@ -261,9 +325,10 @@ class Dapper(suite.Suite):
         return self.vm.distro.install_from_template(*args, **kwargs)
 
     def run_in_target(self, *args, **kwargs):
-        self.vm.distro.run_in_target(*args, **kwargs)
+        return self.vm.distro.run_in_target(*args, **kwargs)
 
     def copy_to_target(self, infile, destpath):
+        logging.debug("Copying %s on host to %s in guest" % (infile, destpath))
         dir = '%s/%s' % (self.destdir, os.path.dirname(destpath))
         if not os.path.isdir(dir):
             os.makedirs(dir)
@@ -280,11 +345,30 @@ class Dapper(suite.Suite):
             os.makedirs('%s/var/lock' % fs.mntpath)
 
     def copy_settings(self):
-        self.copy_to_target('/etc/default/locale', '/etc/default/locale')
-        self.copy_to_target('/etc/timezone', '/etc/timezone')
-        self.run_in_target('dpkg-reconfigure', '-pcritical', 'libc6')
+        if os.path.exists('/etc/default/locale'):
+            self.copy_to_target('/etc/default/locale', '/etc/default/locale')
+        self.run_in_target('dpkg-reconfigure', '-fnoninteractive', '-pcritical', 'libc6')
         self.run_in_target('locale-gen', 'en_US')
         if self.vm.lang:
             self.run_in_target('locale-gen', self.vm.lang)
             self.install_from_template('/etc/default/locale', 'locale', { 'lang' : self.vm.lang })
+        self.run_in_target('dpkg-reconfigure', '-fnoninteractive', '-pcritical', 'locales')
         self.run_in_target('dpkg-reconfigure', '-pcritical', 'locales')
+
+    def install_vmbuilder_log(self, logfile, rootdir):
+        shutil.copy(logfile, '%s/var/log/vmbuilder-install.log' % (rootdir,))
+
+    def set_timezone(self):
+        if self.vm.timezone:
+            os.unlink('%s/etc/localtime' % self.destdir)
+            shutil.copy('%s/usr/share/zoneinfo/%s' % (self.destdir, self.vm.timezone), '%s/etc/localtime' % (self.destdir,))
+
+    def install_ec2(self):
+        if self.vm.ec2:
+            logging.debug('This suite does not support ec2')
+
+    def disable_hwclock_access(self):
+        fp = open('%s/etc/default/rcS' % self.destdir, 'a')
+        fp.write('HWCLOCKACCESS=no')
+        fp.close()
+
