@@ -1,13 +1,12 @@
 #
 #    Uncomplicated VM Builder
-#    Copyright (C) 2007-2008 Canonical Ltd.
+#    Copyright (C) 2007-2009 Canonical Ltd.
 #    
 #    See AUTHORS for list of contributors
 #
 #    This program is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    it under the terms of the GNU General Public License version 3, as
+#    published by the Free Software Foundation.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,6 +19,7 @@
 import logging
 import suite
 import shutil
+import os
 import VMBuilder.disk as disk
 from   VMBuilder.util import run_cmd
 from   VMBuilder.plugins.ubuntu.dapper import Dapper
@@ -48,15 +48,19 @@ proc                                            /proc           proc    defaults
         return retval
 
     def copy_settings(self):
-        self.copy_to_target('/etc/default/locale', '/etc/default/locale')
-        shutil.rmtree('%s/etc/console-setup' % self.destdir)
-        self.copy_to_target('/etc/console-setup', '/etc/console-setup')
-        self.copy_to_target('/etc/default/console-setup', '/etc/default/console-setup')
-        self.copy_to_target('/etc/timezone', '/etc/timezone')
-        self.run_in_target('dpkg-reconfigure', '-pcritical', 'tzdata')
+        if os.path.exists('/etc/default/locale'):
+            self.copy_to_target('/etc/default/locale', '/etc/default/locale')
+        csdir = '%s/etc/console-setup' % self.destdir
+        have_cs = os.path.isdir(csdir)
+        if have_cs:
+            shutil.rmtree(csdir)
+            self.copy_to_target('/etc/console-setup', '/etc/console-setup')
+            self.copy_to_target('/etc/default/console-setup', '/etc/default/console-setup')
+        self.run_in_target('dpkg-reconfigure', '-fnoninteractive', '-pcritical', 'tzdata')
         self.run_in_target('locale-gen', 'en_US')
         if self.vm.lang:
             self.run_in_target('locale-gen', self.vm.lang)
             self.install_from_template('/etc/default/locale', 'locale', { 'lang' : self.vm.lang })
-        self.run_in_target('dpkg-reconfigure', '-pcritical', 'locales')
-        self.run_in_target('dpkg-reconfigure', '-pcritical', 'console-setup')
+        self.run_in_target('dpkg-reconfigure', '-fnoninteractive', '-pcritical', 'locales')
+        if have_cs:
+            self.run_in_target('dpkg-reconfigure', '-fnoninteractive', '-pcritical', 'console-setup')
