@@ -1,13 +1,12 @@
 #
 #    Uncomplicated VM Builder
-#    Copyright (C) 2007-2008 Canonical Ltd.
+#    Copyright (C) 2007-2009 Canonical Ltd.
 #    
 #    See AUTHORS for list of contributors
 #
 #    This program is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    it under the terms of the GNU General Public License version 3, as
+#    published by the Free Software Foundation.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +17,7 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #    Various utility functions
+import errno
 import logging
 import os
 import os.path
@@ -98,12 +98,19 @@ def run_cmd(*argv, **kwargs):
         logging.debug('stdin was set and it was a string: %s' % (stdin,))
         stdin_arg = subprocess.PIPE
     else:
-        stdin_arg = file('/dev/null', 'w')
+        stdin_arg = file('/dev/null', 'r')
     proc_env = dict(os.environ)
     proc_env['LANG'] = 'C'
     proc_env['LC_ALL'] = 'C'
     proc_env.update(env)
-    proc = subprocess.Popen(args, stdin=stdin_arg, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=proc_env)
+
+    try:
+        proc = subprocess.Popen(args, stdin=stdin_arg, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=proc_env)
+    except OSError, error:
+        if error.errno == errno.ENOENT:
+            raise VMBuilderUserError, "Couldn't find the program '%s' on your system" % (argv[0])
+        else:
+            raise VMBuilderUserError, "Couldn't launch the program '%s': %s" % (argv[0], error)
 
     if stdin:
         proc.stdin.write(stdin)
@@ -169,8 +176,8 @@ def render_template(plugin, vm, tmplname, context=None):
         searchList.append(context)
     searchList.append(vm)
 
-    tmpldirs = ['VMBuilder/plugins/%s/templates',
-                os.path.expanduser('~/.vmbuilder/%s'),
+    tmpldirs = [os.path.expanduser('~/.vmbuilder/%s'),
+                os.path.dirname(__file__) + '/plugins/%s/templates',
                 '/etc/vmbuilder/%s']
 
     if vm.templates:
